@@ -2,7 +2,8 @@ from .version import __version__ # noqa
 
 from .datawidgets import (type2Widget, BoolCheckBox, IntLineEdit,
     FloatLineEdit, StrLineEdit, TupleGroupBox)
-from .dataclass_widget import DataclassWidget
+from .dataclass_widget import (DataclassWidget, StackedDataclassWidget,
+    TabDataclassWidget)
 
 
 __all__ = [
@@ -14,4 +15,6 @@ __all__ = [
     "TupleGroupBox",
 
     "DataclassWidget",
+    "StackedDataclassWidget",
+    "TabDataclassWidget",
 ]
